declare module BA.Ui.Controls {
    /** Own interface for both of the containers that the search control contains (indexes and results) */
    interface BAContainerDivElement extends HTMLDivElement {
        cpHighlighted: BAEntryDivElement;
        cpSelected: BAEntryDivElement;
    }
    /** Own interface for list entries of both lists (base interface) */
    interface BAEntryDivElement extends HTMLDivElement {
        cpParentContainer: ContainerBase;
    }
    /** Interface for entries in the index-list */
    interface BAIndexEntry extends BAEntryDivElement {
        cpConfigId: BA.Classes.Guid;
        cpConfigTitle: string;
        cpCount: string;
        cpLngCount: number;
        cpInfoText: string;
        cpOrmType: BA.Classes.Guid;
        cpShowAddToFolder: boolean;
    }
    /** Interface for entires in the results-list */
    interface BAResultEntry extends BAEntryDivElement {
        cpOid: BA.Classes.Guid;
        cpFormOid: BA.Classes.Guid;
        cpOrmType: BA.Classes.Guid;
        cpOpenBehavior: BA.Classes.Guid;
    }
    /** Extension for the search input field */
    interface BASearchFieldInputElement extends HTMLInputElement {
        cpWrapper: SearchControl;
    }
    /**
     * This class is the main control class that applies any needed functionality to the given control complex.
     * It looks up all needed components (divs and search field), initializes functionality and adds listeners as
     * needed. It provides all necessary means for navigating the indexes and the search results by arrow-keys as well
     * as by mouse and opening search results.
     **/
    export class SearchControl {
        /** The HTML input element where the user enters the search term */
        searchField: BASearchFieldInputElement;
        /** The search term, which the user has entered. This is current at all times. */
        searchTerm: string;
        /** The search indexes in which the control searches. */
        searchIndexes: string;
        /** The parent div container for the two divs containing indexes and search results. */
        searchPanel: HTMLDivElement;
        /** The container that contains a navigatable list of configured search indexes. */
        indexContainer: IndexContainer;
        /** The container that contains a navigatable list of search results based on the keywords that the user entered. */
        resultsContainer: ResultsContainer;
        /** The container that contains a navigatable list of search results based on the keywords that the user entered. */
        infoContainer: InfoContainer;
        /** The container that is currently being navigated by arrow-keys. */
        activeContainer: ContainerBase;
        /** True, if the active container should be switched to the other container after the next update. */
        changeContainerAfterUpdate: boolean;
        /** Contains the time stamp when the search field focus element has occurred last */
        lastFocusEvent: number;
        /** Timeout handle for the blur event of the search field */
        blurTimeout: number;
        /**
         * Constructor of this class.
         * It initializes the search field and its containers by looking them up, applying css classes and adding
         * different types of listeners to the search field.
         *
         * @param searchFieldContainer The main div of the search control.
         */
        constructor(searchFieldContainer: HTMLDivElement);
        /**
         * Create the search panel Element
         * */
        private CreateSearchPanel;
        /**
         * If the search panel goes beyond the browser's viewport, the panel is repositioned. If the panel itself
         * does not fit into the current window's margins, the given parameter is used in order to determine at which
         * side of the browser the search panel will be aligned. This might be subject to change in the future.
         *
         * @param ratherRight if true and the panel does not fit entirly, it is aligned with the right margin
         */
        private RepositionSearchPanelAtMargins;
        /**
         * Recalculates the height of the search result panel as follows:
         * - There is a minimum height of 150px.
         * - The preferred height is the full height of the list of search configurations.
         * - If the preferred height would exceed the height of the view port, the height will be cut there (but at least 150px).
         */
        private RecalcSearchPanelHeight;
        private _ParallelSearchCount;
        /**
         *  This is called when the search term changes or the field is focussed in a way that makes the search panel
         *  appear. It fetches information about the configured indexes and the contained number of search results
         *  from a server controller, re-sets the currently selected index entry and by that triggers re-searching
         *  of the search results.
         */
        private UpdateSearch;
        /**
         * Onclick event handler for the search field (also used for onfocus).
         * This event is used for showing the search panel if the current search term has more than two letters.
         * Current highlightings of entries are removed and serach is triggered in order to update the results.
         *
         * @param evt the event
         */
        private OnSearchClick;
        /**
         * Onkeyup event handler for the search field. This handles live-update to the search panel and makes sure that
         * the stored search term always is current.
         *
         * @param evt the event
         */
        private OnSearchKeyUp;
        /**
         * Onkeydown event handler for the search field. This intercepts the navigation by arrow keys, the enter key
         * and finally every key if the pseudo-blur mode is active.
         * In pseudo-blur mode the search field appears as it has lost focus (as the user navigates the search panel)
         * with cursor keys or mouse pointer) but actually only the caret is hidden and all input is rejected in order
         * to be able to react to the arrow keys (which would not be possible if the field lost focus for real).
         *
         * @param evt the event
         */
        private OnSearchKeyDown;
        /**
         * Retrieve search results from server for the given (selected) index and create the ui entries for them.
         *
         * @param forIndex the index to get the search results for
         */
        CreateResults(forIndex: BAIndexEntry): void;
        /**
         * Event function that is called when the "add to folder" link is clicked. It invokes the add-to-folder
         * ui-functionality.
         *
         * @param configId The id of the currently active index.
         * @param ormType The orm type of the contained records.
         */
        OnAddToRecordCollectionClicked(configId: BA.Classes.Guid, ormType: BA.Classes.Guid): void;
        /**
         * Invoked when an index entry is clicked. Changes highlight and selection.
         *
         * @param evt the event
         */
        OnIndexClicked(evt: MouseEvent): void;
        /**
         * Invoked when a result entry is clicked. Opens the target result.
         *
         * @param evt the event
         */
        OnResultClicked(evt: MouseEvent): boolean;
        /**
         * Event that is called when the clear button of the search field was clicked
         *
         * @param context the search control
         */
        OnCleared(context: SearchControl): void;
        /**
         * Opens the given result entry as defined by the entry.
         *
         * @param entry the entry to open
         */
        OpenResult(entry: BAResultEntry): void;
        /**
         * Opens a message box containing excessive help for the syntax that can be used for searching.
         **/
        static OpenSearchHelp(): void;
    }
    /**
     * This is the abstract base class for index- and results-container. It provides common functionality that is
     * shared by both types of containers.
     **/
    abstract class ContainerBase {
        /** The parent search field object */
        protected context: SearchControl;
        /** The extended html div container object */
        container: BAContainerDivElement;
        /**
         * Constructor - stores the given values in member variables.
         *
         * @param context the parent serach field
         * @param container the physical container (div) for this object
         */
        constructor(context: SearchControl, container: BAContainerDivElement);
        /**
         * Checks whether the given element is completely visible on the screen and not scrolled out of view.
         *
         * @param element element to check visibility for
         */
        CheckInView(element: HTMLDivElement): boolean;
        /**
         * Switches the currently highlighted entry to a new one (by arrow-key-navigation or mouse-hover).
         * Also switches the search field into pseudo-blur mode so the search term cannot be changed during navigation.
         * Restores real focus to the search field if the given element is null.
         *
         * @param newElement the new entry to be highlighted
         */
        ChangeHighlight(newElement: HTMLDivElement, byMouseEvent?: boolean): void;
        /**
         * Switches the currently selected entry to a new one (by pressing enter on a highlighted entry or mouse click).
         * Invokes the OnSelectionChanged 'event' that might be overridden by the inheriting classes
         * .
         * @param newSelection the new entry to be selected
         */
        ChangeSelection(newSelection: HTMLDivElement): void;
        /**
         * Create a basic entry div element from the given information. The created object might be enhanced further
         * by the calling function.
         *
         * @param title the main text of the entry
         * @param onclick name of the method to call when the entry is clicked (on SearchField class only!)
         */
        CreateEntry(title: string, onclick: string): BAEntryDivElement;
        /**
         * Replaces the current with the given entries on this container. In order to avoid ui issues like flickers
         * and such, the current container is cloned, cleared, filled and finally completely replaced.
         *
         * @param entries the entries to show in this container
         */
        SetEntries(entries: BAEntryDivElement[]): void;
        /**
         * Event function that is called when the selection in this container has changed.
         * Might be overridden by inheriting classes.
         *
         * @param oldSelection the previous selection
         * @param newSelection the current selection
         */
        OnSelectionChanged(oldSelection: HTMLDivElement, newSelection: HTMLDivElement): void;
        /**
         * Handles for this container what should happen when the user navigates by pressing the up-arrow.
         **/
        OnArrowUp(): void;
        /**
        * Handles for this container what should happen when the user navigates by pressing the down-arrow.
        **/
        OnArrowDown(): void;
        /**
         * Handles for this container what should happen when the user navigates by pressing the enter key.
         **/
        OnEnter(): void;
    }
    /**
     * This class is the concrete implementation of the container that contains the index entries.
     **/
    class IndexContainer extends ContainerBase {
        /**
         * Constructor - just calls base constructor.
         *
         * @param context the parent serach field
         * @param container the physical container (div) for this object
         */
        constructor(context: SearchControl, container: BAContainerDivElement);
        /**
         * Creates an entry for the index container based on the given model data.
         *
         * @param indexData model that contains the necessary information about the new entry
         */
        CreateIndexEntry(indexData: BA.Ui.Models.SearchConfigModel): BAIndexEntry;
        /**
         * Overrides the parent function in order to define what should happen when the user presses the enter key
         * while navigating in this container.
         **/
        OnEnter(): void;
        /**
         * Overrides the parent function in order to define what should happen when the selection in this container
         * has changed. In this case, the results for the selected index should be shown in the results container.
         *
         * @param oldSelection previous selection
         * @param newSelection new selection
         */
        OnSelectionChanged(oldSelection: HTMLDivElement, newSelection: HTMLDivElement): void;
    }
    /**
    * This class is the concrete implementation of the container that contains the index entries.
    **/
    class ResultsContainer extends ContainerBase {
        /**
         * Constructor - just calls base constructor.
         *
         * @param context the parent serach field
         * @param container the physical container (div) for this object
         */
        constructor(context: SearchControl, container: BAContainerDivElement);
        /**
         * Creates an entry for the result container based on the given model data.
         *
         * @param resultData model that contains the necessary information about the new entry
         */
        CreateResultEntry(resultData: BA.Ui.Models.SearchHitModel): BAResultEntry;
        /**
         * Overrides the parent function in order to define what should happen when the user presses the enter key
         * while navigating in this container.
         **/
        OnEnter(): void;
    }
    class InfoContainer {
        /** The parent search field object */
        private context;
        /** The extended html div container object */
        private container;
        constructor(context: SearchControl, container: BAContainerDivElement);
        SetText(text: string, showAddToFolder: boolean): void;
        /** Shows the info container */
        Show(): void;
        /** Hides the info container */
        Hide(): void;
    }
    export {};
}
